# Requires IPHost build 12338 or later
# Certain parameters values
#
# Executable name:
# C:\Windows\System32\WindowsPowerShell\v1.0\powershell.exe
#
# Several examples of script usage are listed below:
#
# Mandatory command-line parameters:
# -ExecutionPolicy ByPass -File  C:\Scripts\esxi-cim-filter-class-instance.ps1 -HostEsxi $HostIP -UserName $WindowsUser -UserPassword $WindowsPassword 
#
# Per-class parameters
#
# CPU Status: command line
# -ClassName CIM_Processor -CreationClassName OMC_Processor -FieldName CPUStatus
#    Acceptable values: 1,4
#
# Battery status: command line
# -ClassName CIM_Battery -CreationClassName OMC_Battery -FieldName BatteryStatus
#    Warning values: 1,2,4,10,11
#    Down values: 5,13
#
# Ethernet adapter: command line
# -ClassName CIM_EthernetPort -CreationClassName VMware_EthernetPort -FieldName OperationalStatus
#    OK values: 2,3,7,14,16
#
# IDE controller: command line
# -ClassName CIM_Controller -CreationClassName VMware_PCIDevice -ElementName "*IDE Controller*" -FieldName HealthState
#    OK values: 0
#
# FAN device: command line
# -ClassName CIM_Fan -CreationClassName OMC_Fan -ElementName "*FAN name here*" -FieldName HealthState
#    OK values: 5
#    Warning values: 10
#
# Physical memory: command line
# -ClassName CIM_PhysicalMemory -CreationClassName OMC_PhysicalMemory -ElementName "DIMM 0" -FieldName OperationalStatus
#    OK values: 0,2
#    Warning values: 3

# Default command-line parameters; the below line must be first non-comment line of the script
param([string]$HostEsxi = "",
	[string]$UserName = "",
	[string]$UserPassword = "",
	[string]$ClassName = "",
	[string]$CreationClassName = "",
	[string]$ElementName = "",
	[string]$FieldName = ""
);

Import-Module CimCmdlets

# Script name (without path)
$SelfName = $MyInvocation.MyCommand.Name
# Sanity checks
if (($HostEsxi -eq "") -or ($UserName -eq "") -or ($UserPassword -eq "") -or ($ClassName -eq "")) {
	Write-host "$($SelfName) parameters expected, i.e. `n    $($SelfName) -HostEsxi ipaddr -UserName root -UserPassword password -ClassName CIM_LogicalDevice -CreationClassName subclass [-ElementName DeviceName] -FieldName metric"
	Exit 1
}
# Encrypt plain password passed
$HostPassword = $UserPassword | ConvertTo-SecureString -AsPlaintext -Force

# Authenticate and create session for ESXi host
$Credentials = New-object -typename System.Management.Automation.PSCredential -argumentlist $UserName, $HostPassword
$CIOpt = New-CimSessionOption -SkipCACheck -SkipCNCheck -SkipRevocationCheck -Encoding Utf8 -UseSsl
$Session = New-CimSession -Authentication Basic -Credential $Credentials -ComputerName $HostEsxi -port 443 -SessionOption $CIOpt
# If -ElementName set, filter by element name
If ($ElementName -eq "") {
	$RC = Get-CimInstance -CimSession $Session -ClassName $ClassName | Where {$_.CreationClassName -eq $CreationClassName}
} else {
	$RC = Get-CimInstance -CimSession $Session -ClassName $ClassName | Where {$_.CreationClassName -eq $CreationClassName} | Where {$_.ElementName -Like $ElementName}
}
# Sanity checks
if ($RC) {
	if ($RC.PSObject.Properties.Match($FieldName).Count) {
		$RC.$FieldName
		Exit 0
	} else {
		"Field $FieldName does not exist"
		Exit 2
	}
} else {
	"No data returned for class $ClassName / creation class $CreationClassName"
	Exit 1
}
